
%%  OTG1 - Basic Structure Generator for OT.
%%
%%  expects the following predicates to be defined in other files:
%%
%%  UI:
%%
%%  message(curr_cand,[Input,Candidates]).
%%     reporting on currant candidates
%%
%%  message(init_cand,[Input,Candidates]).
%%     reporting on initial candidates (before any input is consumed
%%
%%  message(final_cand,Configs).
%%     reporting on surviving final candidates
%%
%%  continue.  Dialog that succeeds iff generation is to continue
%%
%%  Theoretical Environment:
%%
%%  ranking(R).
%%    the ranking of the constraints
%%
%%
%%  sit(S0,S1,Out,Marks).
%%    non-advancing state transitions
%%    in state, out state, list of output symbols, marks added to score
%%
%%  adv(S0,Seg,S1,Out,Marks).
%%    advancing state transitions, Seg being the segment advanced-over.
%%
%%





auxiliaries(MState,Output,AStates0,Marks0,AStates,Marks) :-
  aux_names(ANames),
  auxiliaries(ANames,MState,Output,AStates0,Marks0,AStates,Marks).

auxiliaries([AName|RestNames],MState,Output,[AState0|RestStates0],
    Marks0,[AState|RestStates],Marks) :-
  aux(AName,MState,Output,AState0,AState,AMarks),
  append(AMarks,Marks0,Marks1),
  auxiliaries(RestNames,MState,Output,RestStates0,Marks1,RestStates,Marks).

auxiliaries([],_,_,_,M,[],M).


link(res(Input0,S0:A0,Out0,Score0),res(Input,S:A,Out,Score),Ranking) :-
  link1(Input0,Input,S0,S,Output,Marks0),
  auxiliaries(S,Output,A0,Marks0,A,Marks),
  not(hopeless(Marks)),
  update_score(Score0,Marks,Score,Ranking),
  viable(Input,S,A,Score),
  append(Output,Out0,Out).

link1(Input,Input,S0,S,Output,Marks) :-
  sit(S0,S,Output,Marks),
  end_ok(Input,S).

link1([Seg|Input],Input,S0,S,Output,Marks) :-
  adv(S0,Seg,S,Output,Marks).

end_ok(I,end) :- !,
  I=[].
end_ok(_,_).


viable(Input,S,A,Score1) :-
  deja_vu(S,A,Length,Score2),
  length(Input,Length),
  worse_score(Score1,Score2),!,
  fail.
viable(_,_,_,_).

aux_inits([_|Names],[0|Rest]) :-
  aux_inits(Names,Rest).
aux_inits([],[]).


parse([]) :-
  write('Nothing to do -- see ya later'),nl.

parse(Input) :-
  length(Input,L),
  retractall(input_string(_)),
  retractall(input_length(_)),
  assert(input_string(Input)), % progress-report into
  assert(input_length(L)),   % progress-report info
  ranking(R),!,
  retractall(deja_vu(_,_,_,_)),
  retractall(top_score(_)),
  create_score(R,Score),
  aux_names(Names),
  aux_inits(Names,A),
  parse([res(Input,0:A,[],Score)],R).


parse(_) :-
  message(parse_noranking,[]).

parse([Head|L],Ranking) :-
  Head = res([],end:A,_,Score),
  final_aux(A),
  allright(Score),!,
  message(final_output,Head),
 % write('An Output: '),
  assert(top_score(Score)),
 % writeout([Head]),
  parse(L,Ranking).


parse([Optimal|Remainders],Ranking) :-
  Optimal=res(Input,S:A,_,Score),
  allright(Score),
  length(Input,Length),
  assert(deja_vu(S,A,Length,Score)),
%length(Remainders,Rlength),write('List Length: '),write(Rlength),nl,
  message(curr_cand,Optimal),
  continue,!,
  findall(Result,link(Optimal,Result,Ranking),Results),
  deposit_results(Results,Remainders,Candidates),
  parse(Candidates,Ranking).

parse(_,_) :-
  message('alldone',[]).

final_aux([]).
final_aux([end|A]) :-
  final_aux(A).


allright(Score) :-
  top_score(Score2),!,
  not(worse_score(Score,Score2)).
allright(_).


score(res(_,_,_,Score),Score).

deposit_results([Result|Rest],Candidates0,Candidates) :-
  deposit_result(Result,Candidates0,Candidates1),
  deposit_results(Rest,Candidates1,Candidates).
deposit_results([],Candidates,Candidates).


deposit_result(Result,[Head|Rest],[Result,Head|Rest]) :-
  score(Result,Score1),
  score(Head,Score2),
  worse_score(Score2,Score1),!,
  message(accepting,Result).
%   length([Head|Rest],Length),write('Tail Length: '),write(Length),nl.
deposit_result(Result,[Head|Rest],[Head|Rest]) :-
  useless_result(Result,Head),!.
deposit_result(Result,[Head|Rest0],[Head|Rest]) :- !,
  deposit_result(Result,Rest0,Rest).
deposit_result(Result,[],[Result]) :-
  message(accepting,Result).
   

useless_result(Cand1,Cand2) :-
 Cand1 = res(I,S,Out1,Score1),
 Cand2 = res(I,S,Out2,Score2),
 worse_score(Score1,Score2),!,
 message(worse_score,[Cand1,Cand2]).

useless_result(res(I,S,Out,_),res(I,S,Out,_)).


hopeless(Marks) :-
  member(Mark,Marks),
  undominated(Mark).



create_score([H|R],[0|S]) :-
  create_score(R,S).
create_score([],[]).

count_mark(Mark,[Mark|Rest],List,Number) :- !,
  count_mark(Mark,Rest,List,N0),
  Number is N0+1.
count_mark(Mark,[Head|Rest0],[Head|Rest],Number) :- !,
  count_mark(Mark,Rest0,Rest,Number).
count_mark(_,[],[],0).
  

update_score([Head0|Rest0],Marks,[Head|Rest],[Mark|RestRank]) :- !,
  count_mark(Mark,Marks,RestMarks,Number),
  Head is Head0+Number,
  update_score(Rest0,RestMarks,Rest,RestRank).

update_score([],_,[],[]).

worse_score([N|Rest0],[N|Rest]) :- !,
  worse_score(Rest0,Rest).
worse_score([N|_],[M|_]) :-
  N>M.

write_score([N],[M]) :-
  write(M),write('/'),write(N).
write_score([N|S],[M|R]) :-
  write(M),write('/'),write(N),write(', '),
  write_score(S,R).
write_score([],[]).
