/*

                    Generator for Lardil (P&S ch 7).
                               

segments:  a,e,i,u
           p,m,th,nh,t,n,rr,l,rt,rn,r,rl,ty,ny,k,ng,y,w
             >s (formative boundary)

constraints: align_l, ons, contig_m, wd_bin, coda_cond, fillnuc,
             free_v, align, fillons, parse, nocod

see lardil.txt for discussion of constraints and ranking

known deficiencies:
  POA-assimilation not implemented for post-nonnasal augmentation
  no simplification of `rr t' and `t t'
  cross-morpheme vowel sequences not correctly handled

References:

  Hale, Kenneth (1973) `Deep-Surface Canonical Disparities in
    Relation to Analysis and Change', in Sebeok (ed) _Current
    Trends in Linguistics_, vol 11:401-458.

  Prince, Alan and Paul Smolensky (1993) _Optimality Theory: Constraint
     Interaction in Generative Grammar_, to appear, MIT Press; TR-2,
     Rutgers University Cognitive Science Center.

  Wilkinson, Karina (1988) `Prosodic Structure and Lardil Phonology',
    LI 19:325-334.

*/ 


vowel(a).
vowel(e).
vowel(i).
vowel(u).

glide(y).
glide(w).

conson(p).  place(p,lab).   man(p,stop).
conson(m).  place(m,lab).   man(m,nas).

conson(th). place(th,lam).  man(th,stop).
conson(nh). place(nh,lam).  man(nh,nas).

conson(t).  place(t,cor).   man(t,stop).
conson(n).  place(n,cor).   man(n,nas).
conson(rr). place(rr,cor).  man(rr,rhot).
conson(l).  place(l,cor).   man(l,lat).

conson(rt). place(rt,retr). man(rt,stop).
conson(rn). place(rn,retr). man(rn,nas).
conson(r).  place(r,retr).  man(r,rhot).
conson(rl). place(rl,retr). man(rl,lat).

conson(ty). place(ty,pal).  man(ty,stop).
conson(ny). place(ny,pal).  man(ny,mas).

conson(k).  place(k,vel).   man(k,stop).
conson(ng). place(ng,vel).  man(ng,nas).

conson('N').                man('N',nas).

coda_ok(Seg,[]) :-
  place(Seg,P),
  coronal(P),!.
coda_ok(Seg,[coda_cond]).

coronal(cor) :- !.
coronal(retr) :- !.
coronal(pal) :- !.


/*     AUXILIARY AUTOMATA  */

aux_names([free_v,wd_bin,align_l,contig_m,align]).


aux(free_v,end,_,1,end,[free_v]) :- !. % terminate with free_v violation
aux(free_v,end,_,_,end,[]) :- !.  % allows termination

aux(free_v,_, ['>s'],  _, end, []) :- !.   % shuts off after stem boundary
aux(free_v,_, _,       end,end,[]) :- !.   % stay asleep

aux(free_v,_:x, [nuc(_)], S, 1,   []) :-     % wake up at nonep. vowel 
  not(S=end),!.
aux(free_v,0:_,  [],      1, 1,   []) :- !.  % stay awake at 2->0 trans.

aux(free_v,_,_,_,0,[]).           % back to sleep


aux(wd_bin,end,_,2,end,[]) :- !.       % 2 nuclei seen, end ok
aux(wd_bin,end,_,_,end,[wd_bin]) :- !. % otherwise violation.

aux(wd_bin,_,[nuc(X)],0,1,[]) :- !.    % see a nuc, advance one state.
aux(wd_bin,_,[nuc(X)],1,2,[]) :- !.    % see another, ditto.

aux(wd_bin,_,_,S,S,[]).                   % else do nothing.


aux(align_l,_:ep,  _, 0,end,[align_l]) :- !.
aux(align_l,_:skip,_, 0,end,[align_l]) :- !.
aux(align_l,_:x,   _, 0,end,[]) :- !.
aux(align_l,_,     _, S,S,[]).

% inhbits epenthesis and underparsing within a morpheme.

aux(contig_m,end,_,_,end,[]) :- !.        % end anytime
aux(contig_m,_:x, _, 0,1,[]) :- !.        % wake up at real output
aux(congig_m,_,  '>s',X,0,[]) :- !.       % reset at a boundary
aux(contig_m,_:ep,_, 1,2,[]) :- !.        % hi readiness !!
aux(contig_m,_:skip,_,1,2,[]) :- !.       % ditto
aux(contig_m,_:x,_,2,0,[contig_m]) :- !.  % detect violation
aux(contig_m,_,_,S,S,[]).


/* align is supposed to register a violation when the final (overt) segment
   of a formative is not the final segment of a syllable.  There
   are three states; 0, (asleep) where a violation is not going to be
   registered when the end or a boundary is reached, and 1(0) (awake),
   where one will be.  A faithful output that is not syllable final
   always kicks into state 1.
*/

aux(align, _:skip,_,_,1, []) :- !.  % After a skip, alert
aux(align, 1:x,_,  _,1,  []) :- !.  % After a faithful onset, alert
aux(align, 2:x,_,  _,10, []) :- !.  % After a faithful nucleus, alert b.
aux(align, _:x,_,  _,0,  []) :- !.  % Any other faithful output is
                                    %  syllable-final; back to sleep!
aux(align, 0:zip,[],  10,0,  []) :- !.  % if nucleus final in syll. sleep!

aux(align, end,_,  0,end, []) :- !.       % end while asleep, no problems
aux(align, end,_,  _,end, [align]) :- !.  % otherwise, violation!

aux(align, _,'>s', S,0,   [align]) :-   % see boundary while awake, whinge!
  not(S=0),!.                           %   then conk out again.
aux(align, _,_,    S,S,  []).        % otherwise, preserve state.


ok_ons(Seg) :-
  conson(Seg),!.
ok_ons(Seg) :-
  glide(Seg),!.

sit(0,0:zip,[],[]).

adv(S:X,'>s',S:X,['>s'],[]) :- !.      % pass stem-boundary into output

adv(S:X,Seg,S:skip,[],[parse]).      % skip, incurring a parse violation

adv(0:X,Seg,1:x,[ons(Seg)],[]) :-    % parse a consonant as onset, penalty-free.
  ok_ons(Seg).

sit(0:X,1:ep,[ons(nil)],[fillons,fillpoa]).      % epenthetic onset, no place

sit(0:X,1:X,[],[ons]).               % no onset

adv(1:X,Seg,2:x,[nuc(Seg)],[]) :-    % parse a vowel as nucleus, penalty-free.
 vowel(Seg).

sit(1:X,2:ep,[nuc(nil)],[fillnuc]).      % epenthetic nucleus
  
sit(2:X,0:zip,[],[]).                  % next syllable, no coda

adv(2:X,Seg,0:x,[cod(Seg)],[nocod,coda_place | Coda_Cond]):- % parse coda
  conson(Seg),
  coda_ok(Seg,Coda_Cond).


adv(2:X,Seg,ass(Place):x,[cod(Seg)],[nocod]) :-
  conson(Seg),
  man(Seg,nas),
  place(Seg,Place).


adv(ass(Place):x,Seg,1:x,[ons(Seg)],[]) :-    % parse a consonant as onset, penalty-free.
  conson(Seg),
  place(Seg,Place).

sit(ass(Place):X,1:ep,[ons(Seg)],[fillons]) :- % assimilated epenthesis
  man(Seg,stop),                               %  no fillplace violation
  place(Seg,Place).


sit(0:X,end,[],[]).                % OK to end



%%  Structure Display.  Converts the output list to a more attractive
%%   format - here just reversing it (internal rep is last-processed-first).

typography(In,Out) :-
  reverse(In,Out).
